searchNodes=[{"doc":"Client-driven interactive widgets for Livebook. Kino is the library used by Livebook to render rich and interactive outputs directly from your Elixir code. Built-in kinos Kino renders any data structure that implements the Kino.Render protocol, falling back to the Kernel.inspect/2 representation whenever an implementation is not available. The data structures supported by Kino out of the box are: Kino.DataTable Kino.DataTable implements a data table output for user-provided tabular data: data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] Kino.DataTable . new ( data ) Kino.ETS Kino.ETS implements a data table output for ETS tables in the system: tid = :ets . new ( :users , [ :set , :public ] ) Kino.ETS . new ( tid ) Kino.Image Kino.Image wraps binary image content and can be used to render raw images of any given format: content = File . read! ( &quot;/path/to/image.jpeg&quot; ) Kino.Image . new ( content , &quot;image/jpeg&quot; ) Kino.Markdown Kino.Markdown wraps Markdown content for richer text rendering. Kino.Markdown . new ( &quot;&quot;&quot; # Example A regular Markdown file. ## Code ```elixir &quot;Elixir&quot; |&gt; String.graphemes() |&gt; Enum.frequencies() ``` ## Table | ID | Name | Website | | -- | ------ | ----------------------- | | 1 | Elixir | https://elixir-lang.org | | 2 | Erlang | https://www.erlang.org | &quot;&quot;&quot; ) Kino.Frame Kino.Frame is a placeholder for static outputs that can be dynamically updated. frame = Kino.Frame . new ( ) |&gt; Kino . render ( ) for i &lt;- 1 .. 100 do Kino.Frame . render ( frame , i ) Process . sleep ( 50 ) end Also see Kino.animate/3 . User interactions Kino.Input and Kino.Control provide a set of widgets for entering data and capturing user events. See the respective module documentation for examples. All others All other data structures are rendered as text using Elixir's Kernel.inspect/2 . Custom kinos Kino makes it possible to define custom JavaScript powered kinos, see Kino.JS and Kino.JS.Live for more details.","ref":"Kino.html","title":"Kino","type":"module"},{"doc":"Returns a kino that periodically calls the given function to render a new result. The callback is run every interval_ms milliseconds and receives the accumulated value. The callback should return either of: {:cont, term_to_render, acc} - the continue :halt - to no longer schedule callback evaluation This function uses Kino.Frame as the underlying kino. Examples # Render new Markdown every 100ms Kino . animate ( 100 , 0 , fn i -&gt; md = Kino.Markdown . new ( &quot;**Iteration: ` \#{ i } `**&quot; ) { :cont , md , i + 1 } end )","ref":"Kino.html#animate/3","title":"Kino.animate/3","type":"function"},{"doc":"Configures Kino. The supported options are: :inspect They are discussed individually in the sections below. Inspect A keyword list containing inspect options used for printing usual evaluation results. Defaults to pretty formatting with a limit of 50 entries. To show more entries, you configure a higher limit: Kino . configure ( inspect : [ limit : 200 ] ) You can also show all entries by setting the limit to :infinity , but keep in mind that for large data structures it is memory-expensive and is not an advised configuration in this case. Instead prefer the use of IO.inspect/2 with :infinity limit when needed. See Inspect.Opts for the full list of options.","ref":"Kino.html#configure/1","title":"Kino.configure/1","type":"function"},{"doc":"Inspects the given term as cell output. This works essentially the same as IO.inspect/2 , except it always produces colored text and respects the configuration set with configure/1 . Opposite to render/1 , it does not attempt to render the given term as a kino.","ref":"Kino.html#inspect/2","title":"Kino.inspect/2","type":"function"},{"doc":"Returns a special value that results in no visible output.","ref":"Kino.html#nothing/0","title":"Kino.nothing/0","type":"function"},{"doc":"Renders the given term as cell output. This effectively allows any Livebook cell to have multiple evaluation results.","ref":"Kino.html#render/1","title":"Kino.render/1","type":"function"},{"doc":"Starts a process under the Kino supervisor. The process is automatically terminated when the current process terminates or the current cell reevaluates.","ref":"Kino.html#start_child/1","title":"Kino.start_child/1","type":"function"},{"doc":"","ref":"Kino.html#t:nothing/0","title":"Kino.nothing/0","type":"type"},{"doc":"A kino for interactively viewing tabular data. The data must be a tabular data supported by Table . Examples data = [ %{ id : 1 , name : &quot;Elixir&quot; , website : &quot;https://elixir-lang.org&quot; } , %{ id : 2 , name : &quot;Erlang&quot; , website : &quot;https://www.erlang.org&quot; } ] Kino.DataTable . new ( data ) The tabular view allows you to quickly preview the data and analyze it thanks to sorting capabilities. data = Process . list ( ) |&gt; Enum . map ( &amp; Process . info / 1 ) Kino.DataTable . new ( data , keys : [ :registered_name , :initial_call , :reductions , :stack_size ] )","ref":"Kino.DataTable.html","title":"Kino.DataTable","type":"module"},{"doc":"Creates a new kino displaying given tabular data. Options :keys - a list of keys to include in the table for each record. The order is reflected in the rendered table. Optional :name - The displayed name of the table. Defaults to &quot;Data&quot; :sorting_enabled - whether the table should support sorting the data. Sorting requires traversal of the whole enumerable, so it may not be desirable for large lazy enumerables. Defaults to true","ref":"Kino.DataTable.html#new/2","title":"Kino.DataTable.new/2","type":"function"},{"doc":"","ref":"Kino.DataTable.html#t:t/0","title":"Kino.DataTable.t/0","type":"type"},{"doc":"A kino for interactively viewing an ETS table. Examples tid = :ets . new ( :users , [ :set , :public ] ) Kino.ETS . new ( tid ) Kino.ETS . new ( :elixir_config )","ref":"Kino.ETS.html","title":"Kino.ETS","type":"module"},{"doc":"Creates a new kino displaying the given ETS table. Note that private tables cannot be read by an arbitrary process, so the given table must have either public or protected access.","ref":"Kino.ETS.html#new/1","title":"Kino.ETS.new/1","type":"function"},{"doc":"","ref":"Kino.ETS.html#t:t/0","title":"Kino.ETS.t/0","type":"type"},{"doc":"A placeholder for outputs. A frame wraps outputs that can be dynamically updated at any time. Also see Kino.animate/3 which offers a convenience on top of this kino. Examples frame = Kino.Frame . new ( ) |&gt; Kino . render ( ) for i &lt;- 1 .. 100 do Kino.Frame . render ( frame , i ) Process . sleep ( 50 ) end Or with a scheduled task in the background. frame = Kino.Frame . new ( ) |&gt; Kino . render ( ) Kino.Frame . periodically ( frame , 50 , 0 , fn i -&gt; Kino.Frame . render ( frame , i ) { :cont , i + 1 } end )","ref":"Kino.Frame.html","title":"Kino.Frame","type":"module"},{"doc":"Renders and appends the given term to the frame.","ref":"Kino.Frame.html#append/2","title":"Kino.Frame.append/2","type":"function"},{"doc":"Removes all outputs within the given frame.","ref":"Kino.Frame.html#clear/1","title":"Kino.Frame.clear/1","type":"function"},{"doc":"Creates a new frame.","ref":"Kino.Frame.html#new/0","title":"Kino.Frame.new/0","type":"function"},{"doc":"Registers a callback to run periodically in the frame process. The callback is run every interval_ms milliseconds and receives the accumulated value. The callback should return either of: {:cont, acc} - the continue with the new accumulated value :halt - to no longer schedule callback evaluation The callback is run for the first time immediately upon registration.","ref":"Kino.Frame.html#periodically/4","title":"Kino.Frame.periodically/4","type":"function"},{"doc":"Renders the given term within the frame. This works similarly to Kino.render/1 , but the rendered output replaces existing frame contents.","ref":"Kino.Frame.html#render/2","title":"Kino.Frame.render/2","type":"function"},{"doc":"","ref":"Kino.Frame.html#t:t/0","title":"Kino.Frame.t/0","type":"opaque"},{"doc":"A struct wrapping a binary image. This is just a meta-struct that implements the Kino.Render protocol, so that it gets rendered as the underlying image. Examples content = File . read! ( &quot;/path/to/image.jpeg&quot; ) Kino.Image . new ( content , &quot;image/jpeg&quot; )","ref":"Kino.Image.html","title":"Kino.Image","type":"module"},{"doc":"Wraps the given binary content into the image struct. The given type be either :jpeg , :png , :gif , :svg or a string with image MIME type.","ref":"Kino.Image.html#new/2","title":"Kino.Image.new/2","type":"function"},{"doc":"","ref":"Kino.Image.html#t:common_image_type/0","title":"Kino.Image.common_image_type/0","type":"type"},{"doc":"","ref":"Kino.Image.html#t:mime_type/0","title":"Kino.Image.mime_type/0","type":"type"},{"doc":"","ref":"Kino.Image.html#t:t/0","title":"Kino.Image.t/0","type":"opaque"},{"doc":"A struct wrapping a Markdown content. This is just a meta-struct that implements the Kino.Render protocol, so that it gets rendered as markdown. Examples Kino.Markdown . new ( &quot;&quot;&quot; # Example A regular Markdown file. ## Code ```elixir &quot;Elixir&quot; |&gt; String.graphemes() |&gt; Enum.frequencies() ``` ## Table | ID | Name | Website | | -- | ------ | ----------------------- | | 1 | Elixir | https://elixir-lang.org | | 2 | Erlang | https://www.erlang.org | &quot;&quot;&quot; ) This format may come in handy when exploring Markdown from external sources: content = File . read! ( &quot;/path/to/README.md&quot; ) Kino.Markdown . new ( content )","ref":"Kino.Markdown.html","title":"Kino.Markdown","type":"module"},{"doc":"Wraps the given binary content into the markdown struct.","ref":"Kino.Markdown.html#new/1","title":"Kino.Markdown.new/1","type":"function"},{"doc":"","ref":"Kino.Markdown.html#t:t/0","title":"Kino.Markdown.t/0","type":"opaque"},{"doc":"Various widgets for user interactions. Each widget is a UI control element that the user interacts with, consequently producing an event stream. Those widgets are often useful paired with Kino.Frame for presenting content that changes upon user interactions. Examples First, create a control and make sure it is rendered, either by placing it at the end of a code cell or by explicitly rendering it with Kino.render/1 . button = Kino.Control . button ( &quot;Hello&quot; ) Next, to receive events from the control, a process needs to subscribe to it and specify pick a name to distinguish the events. Kino.Control . subscribe ( button , :hello ) As the user interacts with the button, the subscribed process receives corresponding events. IEx.Helpers . flush ( ) #=&gt; {:hello, %{origin: #PID&lt;10895.9854.0&gt;}} #=&gt; {:hello, %{origin: #PID&lt;10895.9854.0&gt;}}","ref":"Kino.Control.html","title":"Kino.Control","type":"module"},{"doc":"Creates a new button.","ref":"Kino.Control.html#button/1","title":"Kino.Control.button/1","type":"function"},{"doc":"Creates a new form. A form is composed of regular inputs from the Kino.Input module, however in a form input values are not synchronized between users. Consequently, the form is another control for producing user-specific events. Either :submit or :report_changes must be specified. Options :submit - specifies the label to use for the submit button and enables submit events :report_changes - whether to send new form value whenever any of the input changes. Defaults to false :reset_on_submit - a list of fields to revert to their default values once the form is submitted. Use true to indicate all fields. Defaults to [] Event info In addition to standard properties, all events include additional properties. :type - either :submit or :change :data - a map with field values, matching the field list Examples Create a form out of inputs: form = Kino.Control . form ( [ name : Kino.Input . text ( &quot;Name&quot; ) , message : Kino.Input . textarea ( &quot;Message&quot; ) ] , submit : &quot;Send&quot; ) Subscribe to events: Kino.Control . subscribe ( form , :chat_form ) As users submit the form the payload is sent: IEx.Helpers . flush ( ) #=&gt; {:chat_form, #=&gt; %{ #=&gt; data: %{message: &quot;Hola&quot;, name: &quot;Amy&quot;}, #=&gt; origin: #PID&lt;10905.5195.0&gt;, #=&gt; type: :submit #=&gt; }} #=&gt; {:chat_form, #=&gt; %{ #=&gt; data: %{message: &quot;Hey!&quot;, name: &quot;Jake&quot;}, #=&gt; origin: #PID&lt;10905.5186.0&gt;, #=&gt; type: :submit #=&gt; }}","ref":"Kino.Control.html#form/2","title":"Kino.Control.form/2","type":"function"},{"doc":"Returns a new interval event source. This can be used as event source for stream/1 and tagged_stream/1 . The events are emitted periodically with an increasing value, starting from 0 and have the form: %{ type : :interval , iteration : non_neg_integer ( ) }","ref":"Kino.Control.html#interval/1","title":"Kino.Control.interval/1","type":"function"},{"doc":"Creates a new keyboard control. This widget is represented as button that toggles interception mode, in which the given keyboard events are captured. Event info In addition to standard properties, all events include additional properties. Key events :type - either :keyup or :keydown :key - the value matching the browser KeyboardEvent.key Status event :type - either :status :enabled - whether the keyboard is activated Examples Create the widget: keyboard = Kino.Control . keyboard ( [ :keyup , :keydown , :status ] ) Subscribe to events: Kino.Control . subscribe ( keyboard , :keyboard ) As the user types events are streamed: IEx.Helpers . flush ( ) #=&gt; {:keyboard, %{enabled: true, origin: #PID&lt;10895.9854.0&gt;, type: :status} #=&gt; {:keyboard, %{key: &quot;o&quot;, origin: #PID&lt;10895.9854.0&gt;, type: :keydown}} #=&gt; {:keyboard, %{key: &quot;k&quot;, origin: #PID&lt;10895.9854.0&gt;, type: :keydown}} #=&gt; {:keyboard, %{key: &quot;o&quot;, origin: #PID&lt;10895.9854.0&gt;, type: :keyup}} #=&gt; {:keyboard, %{key: &quot;k&quot;, origin: #PID&lt;10895.9854.0&gt;, type: :keyup}}","ref":"Kino.Control.html#keyboard/1","title":"Kino.Control.keyboard/1","type":"function"},{"doc":"Returns a Stream of control events. This is an alternative API to subscribe/2 , such that event messages are consume via stream instead of process messages. It accepts a single source or a list of sources, where each source is either of: Kino.Control - emitting value on relevant interaction Kino.Input - emitting value on value change Example button = Kino.Control . button ( &quot;Hello&quot; ) for event &lt;- Kino.Control . stream ( button ) do IO . inspect ( event ) end #=&gt; %{origin: #PID&lt;10895.9854.0&gt;, type: :click} #=&gt; %{origin: #PID&lt;10895.9854.0&gt;, type: :click} Or with multiple sources: button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) interval = Kino.Control . interval ( 1000 ) for event &lt;- Kino.Control . stream ( [ button , input , interval ] ) do IO . inspect ( event ) end #=&gt; %{type: :interval, iteration: 0} #=&gt; %{origin: #PID&lt;10895.9854.0&gt;, type: :click} #=&gt; %{origin: #PID&lt;10895.9854.0&gt;, type: :change, value: true}","ref":"Kino.Control.html#stream/1","title":"Kino.Control.stream/1","type":"function"},{"doc":"Subscribes the given function to events from one or more event sources. The given function runs asynchronously. See stream/1 for more details on event sources. Examples button = Kino.Control . button ( &quot;Greet&quot; ) Kino.Control . stream ( button , fn _event -&gt; IO . puts ( &quot;Hello!&quot; ) end )","ref":"Kino.Control.html#stream/2","title":"Kino.Control.stream/2","type":"function"},{"doc":"A stateful version of stream/1 . Examples button = Kino.Control . button ( &quot;Greet&quot; ) Kino.Control . stream ( button , 0 , fn _event , counter -&gt; new_counter = counter + 1 IO . puts ( &quot;clicks: \#{ new_counter } &quot; ) new_counter end )","ref":"Kino.Control.html#stream/3","title":"Kino.Control.stream/3","type":"function"},{"doc":"Subscribes the calling process to control or input events. The events are sent as {tag, info} , where info is a map with event details. In particular, it always includes :origin , which is an opaque identifier of the client that triggered the event.","ref":"Kino.Control.html#subscribe/2","title":"Kino.Control.subscribe/2","type":"function"},{"doc":"Same as stream/1 , but attaches custom tag to every stream item. Example button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) for event &lt;- Kino.Control . tagged_stream ( [ hello : button , check : input ] ) do IO . inspect ( event ) end #=&gt; {:hello, %{origin: #PID&lt;10895.9854.0&gt;, type: :click}} #=&gt; {:check, %{origin: #PID&lt;10895.9854.0&gt;, type: :change, value: true}}","ref":"Kino.Control.html#tagged_stream/1","title":"Kino.Control.tagged_stream/1","type":"function"},{"doc":"Same as stream/2 , but attaches custom tag to every event. Example button = Kino.Control . button ( &quot;Hello&quot; ) input = Kino.Input . checkbox ( &quot;Check&quot; ) Kino.Control . tagged_stream ( [ hello : button , check : input ] , fn { :hello , event } -&gt; ... { :check , event } -&gt; ... end )","ref":"Kino.Control.html#tagged_stream/2","title":"Kino.Control.tagged_stream/2","type":"function"},{"doc":"A stateful version of tagged_stream/2 . Example up = Kino.Control . button ( &quot;Up&quot; ) |&gt; Kino . render ( ) down = Kino.Control . button ( &quot;Down&quot; ) |&gt; Kino . render ( ) Kino.Control . tagged_stream ( [ up : up , down : down ] , 0 , fn { :up , _event } , counter -&gt; new_counter = counter + 1 IO . puts ( &quot;counter: \#{ new_counter } &quot; ) new_counter { :down , _event } , counter -&gt; new_counter = counter - 1 IO . puts ( &quot;counter: \#{ new_counter } &quot; ) new_counter end )","ref":"Kino.Control.html#tagged_stream/3","title":"Kino.Control.tagged_stream/3","type":"function"},{"doc":"Unsubscribes the calling process from control or input events.","ref":"Kino.Control.html#unsubscribe/1","title":"Kino.Control.unsubscribe/1","type":"function"},{"doc":"","ref":"Kino.Control.html#t:event_source/0","title":"Kino.Control.event_source/0","type":"type"},{"doc":"","ref":"Kino.Control.html#t:interval/0","title":"Kino.Control.interval/0","type":"opaque"},{"doc":"","ref":"Kino.Control.html#t:t/0","title":"Kino.Control.t/0","type":"opaque"},{"doc":"Various input elements for entering data. Examples First, create an input and make sure it is rendered, either by placing it at the end of a code cell or by explicitly rendering it with Kino.render/1 . input = Kino.Input . text ( &quot;Name&quot; ) Then read the value at any later point: name = Kino.Input . read ( input ) Async API You can subscribe to input changes or use the Stream API for event feed. See the Kino.Control module for more details.","ref":"Kino.Input.html","title":"Kino.Input","type":"module"},{"doc":"Creates a new checkbox. The input value can be either true or false . Options :default - the initial input value. Defaults to false","ref":"Kino.Input.html#checkbox/2","title":"Kino.Input.checkbox/2","type":"function"},{"doc":"Creates a new color input. The input value can be a hex color string. Options :default - the initial input value. Defaults to #6583FF","ref":"Kino.Input.html#color/2","title":"Kino.Input.color/2","type":"function"},{"doc":"Creates a new number input. The input value is can be either a number or nil . Options :default - the initial input value. Defaults to nil","ref":"Kino.Input.html#number/2","title":"Kino.Input.number/2","type":"function"},{"doc":"Creates a new password input. This is similar to text input, except the content is not visible by default. Options :default - the initial input value. Defaults to &quot;&quot;","ref":"Kino.Input.html#password/2","title":"Kino.Input.password/2","type":"function"},{"doc":"Creates a new slider input. The input value can be either float in the configured range. Options :default - the initial input value. Defaults to the minimum value :min - the minimum value :max - the maximum value :step - the slider increment","ref":"Kino.Input.html#range/2","title":"Kino.Input.range/2","type":"function"},{"doc":"Synchronously reads the current input value. Note that to retrieve the value, the input must be rendered first, otherwise an error is raised. Examples input = Kino.Input . text ( &quot;Name&quot; ) Kino.Input . read ( input )","ref":"Kino.Input.html#read/1","title":"Kino.Input.read/1","type":"function"},{"doc":"Creates a new select input. The input expects a list of options in the form [{value, label}] , where value is an arbitrary term and label is a descriptive string. Options :default - the initial input value. Defaults to the first value from the given list of options Examples Kino.Input . select ( &quot;Language&quot; , [ en : &quot;English&quot; , fr : &quot;Français&quot; ] ) Kino.Input . select ( &quot;Language&quot; , [ { 1 , &quot;One&quot; } , { 2 , &quot;Two&quot; } , { 3 , &quot;Three&quot; } ] )","ref":"Kino.Input.html#select/3","title":"Kino.Input.select/3","type":"function"},{"doc":"Creates a new text input. Options :default - the initial input value. Defaults to &quot;&quot;","ref":"Kino.Input.html#text/2","title":"Kino.Input.text/2","type":"function"},{"doc":"Creates a new multiline text input. Options :default - the initial input value. Defaults to &quot;&quot;","ref":"Kino.Input.html#textarea/2","title":"Kino.Input.textarea/2","type":"function"},{"doc":"Creates a new URL input. The input value can be either a valid URL string or nil . Options :default - the initial input value. Defaults to nil","ref":"Kino.Input.html#url/2","title":"Kino.Input.url/2","type":"function"},{"doc":"","ref":"Kino.Input.html#t:t/0","title":"Kino.Input.t/0","type":"opaque"},{"doc":"Allows for defining custom JavaScript powered kinos. Example Here's how we could define a minimal kino that embeds the given HTML directly into the page. defmodule KinoDocs.HTML do use Kino.JS def new ( html ) do Kino.JS . new ( __MODULE__ , html ) end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.root.innerHTML = html; } &quot;&quot;&quot; end end Let's break down the API. To define a custom kino we need to create a new module. In this case we go with KinoDocs.HTML . We start by adding use Kino.JS , which makes our module asset-aware. In particular, it allows us to use the asset/2 macro to define arbitrary files directly in the module source. All kinos require a main.js file that defines a JavaScript module and becomes the entrypoint on the client side. The JavaScript module is expected to export the init(ctx, data) function, where ctx is a special object (discussed in detail later) and data is the kino data passed from the Elixir side. In our example the init function accesses the root element with ctx.root and overrides its content with the given HTML string. Finally, we define the new(html) function that creates kinos with the given HTML. Underneath we call Kino.JS.new/2 specifying our module as the kino type and passing the data (available in the JavaScript init function later). Again, it's a convention for each kino module to define a new function to provide uniform experience for the end user. Assets We already saw how to define a JavaScript (or any other) file using the asset/2 macro, however in most cases it's preferable to put assets in a dedicated directory to benefit from syntax highlighting and other editor features. To do that, we just need to specify where the corresponding directory is located: use Kino.JS , assets_path : &quot;lib/assets/html&quot; Stylesheets The ctx.importCSS(url) function allows us to load CSS from the given URL into the page. The stylesheet can be an external resource, such as a font from Google Fonts or a custom asset (as outlined above). Here's an example of both: defmodule KinoDocs.HTML do use Kino.JS def new ( html ) do Kino.JS . new ( __MODULE__ , html ) end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.importCSS(&quot;https://fonts.googleapis.com/css?family=Sofia&quot;) ctx.importCSS(&quot;main.css&quot;) ctx.root.innerHTML = html; } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; body { font-family: &quot;Sofia&quot;, sans-serif; } &quot;&quot;&quot; end end URLs When using multiple asset files, make sure to use relative URLs. For example, when adding an image to the page, instead of: &lt; img src = &quot;/images/cat.jpeg&quot; / &gt; Do: &lt; img src = &quot;./images/cat.jpeg&quot; / &gt; This will correctly point to the images/cat.jpeg file in your assets. Security Note that all assets are assumed public and Livebook doesn't enforce authentication when loading them. Therefore, never include any sensitive credentials in the assets source, instead pass them as arguments from your Elixir code. JavaScript API In the example we briefly introduced the ctx (context) object that is made available in the init(ctx, data) function. This object encapsulates all of the Livebook-specific API that we can call on the JavaScript side. Properties ctx.root - the root element controlled by the kino Functions ctx.importCSS(url) - loads CSS from the given URL into the page. Returns a Promise that resolves once the CSS is loaded ctx.handleEvent(event, callback) - registers an event handler. Once event is broadcasted, callback is executed with the event payload. This applies to Kino.JS.Live kinos ctx.pushEvent(event, payload) - sends an event to the kino server, where it is handled with Kino.JS.Live.handle_event/3 . This applies to Kino.JS.Live kinos CDN It is possible to use a regular JavaScript bundler for generating the assets, however in many cases a simpler and preferred approach is to import the necessary dependencies directly from a CDN. To give a concrete example, here's how we could use the mermaid JavaScript package for rendering diagrams: defmodule Kino.Mermaid do use Kino.JS def new ( graph ) do Kino.JS . new ( __MODULE__ , graph ) end asset &quot;main.js&quot; do &quot;&quot;&quot; import &quot;https://cdn.jsdelivr.net/npm/mermaid@8.13.3/dist/mermaid.min.js&quot;; mermaid.initialize({ startOnLoad: false }); export function init(ctx, graph) { mermaid.render(&quot;graph1&quot;, graph, (svgSource, bindListeners) =&gt; { ctx.root.innerHTML = svgSource; bindListeners &amp;&amp; bindListeners(ctx.root); }); } &quot;&quot;&quot; end end And we would use it like so: Kino.Mermaid . new ( &quot;&quot;&quot; graph TD; A--&gt;B; A--&gt;C; B--&gt;D; C--&gt;D; &quot;&quot;&quot; ) Live kinos So far we covered the API for defining static kinos, where the JavaScript side only receives the initial data and there is no further interaction with the Elixir side. To introduce such interaction, see Kino.JS.Live as a next step in our discussion.","ref":"Kino.JS.html","title":"Kino.JS","type":"module"},{"doc":"Defines an asset file. This serves as a convenience when prototyping or building simple kinos, otherwise you most likely want to put assets in separate files. See the Assets for more details. Examples asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, data) { ... } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; .box { ... } &quot;&quot;&quot; end","ref":"Kino.JS.html#asset/2","title":"Kino.JS.asset/2","type":"macro"},{"doc":"Instantiates a static JavaScript kino defined by module . The given data is passed directly to the JavaScript side during initialization. Options :export_info_string - used as the info string for the Markdown code block where output data is persisted :export_key - in case the data is a map and only a specific part should be exported Export The output can optionally be exported in notebook source by specifying :export_info_string . For example: data = &quot;graph TD;A--&gt;B;&quot; Kino.JS . new ( __MODULE__ , data , export_info_string : &quot;mermaid&quot; ) Would be rendered as the following Live Markdown: ```mermaid graph TD;A--&gt;B; ``` Non-binary data is automatically serialized to JSON.","ref":"Kino.JS.html#new/3","title":"Kino.JS.new/3","type":"function"},{"doc":"","ref":"Kino.JS.html#t:t/0","title":"Kino.JS.t/0","type":"opaque"},{"doc":"Introduces state and event-driven capabilities to JavaScript powered kinos. Make sure to read the introduction to JavaScript kinos in Kino.JS for more context. Similarly to static kinos, live kinos involve a custom JavaScript code running in the browser. In fact, this part of the API is the same. In addition, each live kino has a server process running on the Elixir side, responsible for maintaining state and able to communicate with the JavaScript side at any time. Again, to illustrate the ideas we start with a minimal example. Example We will follow up on our KinoDocs.HTML example by adding support for replacing the content on demand. defmodule KinoDocs.LiveHTML do use Kino.JS use Kino.JS.Live def new ( html ) do Kino.JS.Live . new ( __MODULE__ , html ) end def replace ( kino , html ) do Kino.JS.Live . cast ( kino , { :replace , html } ) end @impl true def init ( html , ctx ) do { :ok , assign ( ctx , html : html ) } end @impl true def handle_connect ( ctx ) do { :ok , ctx . assigns . html , ctx } end @impl true def handle_cast ( { :replace , html } , ctx ) do broadcast_event ( ctx , &quot;replace&quot; , html ) { :noreply , assign ( ctx , html : html ) } end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, html) { ctx.root.innerHTML = html; ctx.handleEvent(&quot;replace&quot;, (html) =&gt; { ctx.root.innerHTML = html; }); } &quot;&quot;&quot; end end Just as before we define a module, this time calling it KinoDocs.LiveHTML for clarity. Note many similarities to the previous version, we still call use Kino.JS , define the main.js file and define the new(html) function for creating a kino instance. As a matter of fact, the initial result of KinoDocs.LiveHTML.new(html) will render exactly the same as our previous KinoDocs.HTML.new(html) . As for the new bits, we added use Kino.JS.Live to define a live kino server. We use Kino.JS.Live.new/2 for creating the kino instance and we implement a few GenServer -like callbacks. Once the kino server is started with Kino.JS.Live.new/2 , the init/2 callback is called with the initial argument. In this case we store the given html in server state. Whenever the kino is rendered on a new client, the handle_connect/1 callback is called and it builds the initial data for the client. In this case, we always return the stored html . This initial data is then passed to the JavaScript init function. Keep in mind that while the server is initialized once, connect may happen at any point, as the users join/refresh the page. Finally, the whole point of our example is the ability to replace the HTML content directly from the Elixir side and for this purpose we added the public replace(kino, html) function. Underneath the function uses cast/2 to message our server and the message is handled with handle_cast/2 . In this case we store the new html in the server state and broadcast an event with the new value. On the client side, we subscribe to those events with ctx.handleEvent(event, callback) to update the page accordingly. Event handlers You must eventually register JavaScript handlers for all events that the client may receive. However, the registration can be deferred, if the initialization is asynchronous. For example, the following is perfectly fine: export function init(ctx, data) { fetch(data.someUrl).then((resp) =&gt; { ctx.handleEvent(&quot;update&quot;, (payload) =&gt; { // ... }); }); } Or alternatively: export async function init(ctx, data) { const response = await fetch(data.someUrl); ctx.handleEvent(&quot;update&quot;, (payload) =&gt; { // ... }); } In such case all incoming events are buffered and dispatched once the handler is registered. Binary payloads The client-server communication supports binary data, both on initialization and on custom events. On the server side, a binary payload has the form of {:binary, info, binary} , where info is regular JSON-serializable data that can be sent alongside the plain binary. On the client side, a binary payload is represented as [info, buffer] , where info is the additional data and buffer is the binary as ArrayBuffer . The following example showcases how to send and receive events with binary payloads. defmodule Kino.Binary do use Kino.JS use Kino.JS.Live def new ( ) do Kino.JS.Live . new ( __MODULE__ , nil ) end @impl true def handle_connect ( ctx ) do payload = { :binary , %{ message : &quot;hello&quot; } , &lt;&lt; 1 , 2 &gt;&gt; } { :ok , payload , ctx } end @impl true def handle_event ( &quot;ping&quot; , { :binary , _info , binary } , ctx ) do reply_payload = { :binary , %{ message : &quot;pong&quot; } , &lt;&lt; 1 , 2 , binary :: binary &gt;&gt; } broadcast_event ( ctx , &quot;pong&quot; , reply_payload ) { :noreply , ctx } end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, payload) { console.log(&quot;initial data&quot;, payload); ctx.handleEvent(&quot;pong&quot;, ([info, buffer]) =&gt; { console.log(&quot;event data&quot;, [info, buffer]) }); const buffer = new ArrayBuffer(2); const bytes = new Uint8Array(buffer); bytes[0] = 4; bytes[1] = 250; ctx.pushEvent(&quot;ping&quot;, [{ message: &quot;ping&quot; }, buffer]); } &quot;&quot;&quot; end end","ref":"Kino.JS.Live.html","title":"Kino.JS.Live","type":"behaviour"},{"doc":"Makes a synchronous call to the kino server and waits for its reply. See GenServer.call/3 for more details.","ref":"Kino.JS.Live.html#call/3","title":"Kino.JS.Live.call/3","type":"function"},{"doc":"Sends an asynchronous request to the kino server. See GenServer.cast/2 for more details.","ref":"Kino.JS.Live.html#cast/2","title":"Kino.JS.Live.cast/2","type":"function"},{"doc":"Invoked to handle synchronous call/3 messages. See GenServer.handle_call/3 for more details.","ref":"Kino.JS.Live.html#c:handle_call/3","title":"Kino.JS.Live.handle_call/3","type":"callback"},{"doc":"Invoked to handle asynchronous cast/2 messages. See GenServer.handle_cast/2 for more details.","ref":"Kino.JS.Live.html#c:handle_cast/2","title":"Kino.JS.Live.handle_cast/2","type":"callback"},{"doc":"Invoked whenever a new client connects to the server. The returned data is passed to the JavaScript init function of the connecting client.","ref":"Kino.JS.Live.html#c:handle_connect/1","title":"Kino.JS.Live.handle_connect/1","type":"callback"},{"doc":"Invoked to handle client events.","ref":"Kino.JS.Live.html#c:handle_event/3","title":"Kino.JS.Live.handle_event/3","type":"callback"},{"doc":"Invoked to handle all other messages. See GenServer.handle_info/2 for more details.","ref":"Kino.JS.Live.html#c:handle_info/2","title":"Kino.JS.Live.handle_info/2","type":"callback"},{"doc":"Invoked when the server is started. See GenServer.init/1 for more details.","ref":"Kino.JS.Live.html#c:init/2","title":"Kino.JS.Live.init/2","type":"callback"},{"doc":"Instantiates a live JavaScript kino defined by module . The given init_arg is passed to the init/2 callback when the underlying kino process is started.","ref":"Kino.JS.Live.html#new/2","title":"Kino.JS.Live.new/2","type":"function"},{"doc":"Invoked when the server is about to exit. See GenServer.terminate/2 for more details.","ref":"Kino.JS.Live.html#c:terminate/2","title":"Kino.JS.Live.terminate/2","type":"callback"},{"doc":"","ref":"Kino.JS.Live.html#t:payload/0","title":"Kino.JS.Live.payload/0","type":"type"},{"doc":"","ref":"Kino.JS.Live.html#t:t/0","title":"Kino.JS.Live.t/0","type":"opaque"},{"doc":"State available in Kino.JS.Live server callbacks. Properties :assigns - custom server state kept across callback calls :origin - an opaque identifier of the client that triggered the given action. It is set in Kino.JS.Live.handle_connect/1 and Kino.JS.Live.handle_event/3","ref":"Kino.JS.Live.Context.html","title":"Kino.JS.Live.Context","type":"module"},{"doc":"Stores key-value pairs in the state. Examples assign ( ctx , count : 1 , timestamp : DateTime . utc_now ( ) )","ref":"Kino.JS.Live.Context.html#assign/2","title":"Kino.JS.Live.Context.assign/2","type":"function"},{"doc":"Sends an event to the client. The event is dispatched to the registered JavaScript callback on all connected clients. Examples broadcast_event ( ctx , &quot;new_point&quot; , %{ x : 10 , y : 10 } )","ref":"Kino.JS.Live.Context.html#broadcast_event/3","title":"Kino.JS.Live.Context.broadcast_event/3","type":"function"},{"doc":"Updates an existing key with the given function in the state. Examples update ( ctx , :count , &amp; ( &amp;1 + 1 ) )","ref":"Kino.JS.Live.Context.html#update/3","title":"Kino.JS.Live.Context.update/3","type":"function"},{"doc":"","ref":"Kino.JS.Live.Context.html#t:origin/0","title":"Kino.JS.Live.Context.origin/0","type":"type"},{"doc":"","ref":"Kino.JS.Live.Context.html#t:t/0","title":"Kino.JS.Live.Context.t/0","type":"type"},{"doc":"An interface for defining custom smart cells. A smart cell is a UI wizard designed for producing a piece of code that accomplishes a specific task. In other words, a smart cell is like a code template parameterized through UI interactions. This module builds on top of Kino.JS.Live , consequently keeping all of its component and communication mechanics. The additional callbacks specify how the UI maps to source code. Usage Defining a custom cell is similar to writing a regular Kino.JS.Live component, with a couple specifics. First, we only need to define callbacks, so there is no need for using Kino.JS.Live.new/2 . The Kino.JS.Live.init/2 callback always receives attrs/0 as the first argument. Second, we add a few new bits, namely use Kino.SmartCell and the two corresponding callback definitions. Here is an outline of a custom module defmodule Kino.SmartCell.Custom do use Kino.JS use Kino.JS.Live use Kino.SmartCell , name : &quot;Our custom wizard&quot; @impl true def init ( attrs , ctx ) do ... end # Other Kino.JS.Live callbacks ... @impl true def to_attrs ( ctx ) do ... end @impl true def to_source ( attrs ) do ... end end Additionally, in order for Livebook to pick up the custom cell, we need to register our module. This usually happens in application.ex Kino.SmartCell . register ( Kino.SmartCell.Custom ) Example As a minimal example, that's how we can define a cell that allows editing the underlying code directly through a textarea. defmodule Kino.SmartCell.Plain do use Kino.JS use Kino.JS.Live use Kino.SmartCell , name : &quot;Plain code editor&quot; @impl true def init ( attrs , ctx ) do source = attrs [ &quot;source&quot; ] || &quot;&quot; { :ok , assign ( ctx , source : source ) } end @impl true def handle_connect ( ctx ) do { :ok , %{ source : ctx . assigns . source } , ctx } end @impl true def handle_event ( &quot;update&quot; , %{ &quot;source&quot; =&gt; source } , ctx ) do broadcast_event ( ctx , &quot;update&quot; , %{ &quot;source&quot; =&gt; source } ) { :noreply , assign ( ctx , source : source ) } end @impl true def to_attrs ( ctx ) do %{ &quot;source&quot; =&gt; ctx . assigns . source } end @impl true def to_source ( attrs ) do attrs [ &quot;source&quot; ] end asset &quot;main.js&quot; do &quot;&quot;&quot; export function init(ctx, payload) { ctx.importCSS(&quot;main.css&quot;); ctx.root.innerHTML = ` &lt;textarea id=&quot;source&quot;&gt;&lt;/textarea&gt; `; const textarea = ctx.root.querySelector(&quot;#source&quot;); textarea.value = payload.source; textarea.addEventListener(&quot;blur&quot;, (event) =&gt; { ctx.pushEvent(&quot;update&quot;, { source: event.target.value }); }); ctx.handleEvent(&quot;update&quot;, ({ source }) =&gt; { textarea.value = source; }); } &quot;&quot;&quot; end asset &quot;main.css&quot; do &quot;&quot;&quot; #source { box-sizing: border-box; width: 100%; min-height: 100px; } &quot;&quot;&quot; end end And then we would register it as Kino.SmartCell . register ( Kino.SmartCell.Plain ) Collaborative editor If a smart cell requires editing some code (like SQL), it may use a dedicated editor instance managed by Livebook. The editor handles syntax highlighting and collaborative editing, similarly to the built-in cells. To enable the editor, we need to include :editor configuration in options returned from the Kino.JS.Live.init/2 callback. @impl true def init ( attrs , ctx ) do # ... { :ok , ctx , editor : [ attribute : &quot;code&quot; , language : &quot;elixir&quot; ] } end Options :attribute - the key to put the source text under in attrs . Required :language - the editor language, used for syntax highlighting. Defaults to nil :placement - editor placement within the smart cell, either :top or :bottom . Defaults to :bottom :default_source - the initial editor source. Defaults to &quot;&quot; Other options Other than the editor configuration, the following options are supported: :reevaluate_on_change - if the cell should be reevaluated whenever the generated source code changes. This option may be helpful in cases where the cell output is a crucial element of the UI interactions. Defaults to false","ref":"Kino.SmartCell.html","title":"Kino.SmartCell","type":"behaviour"},{"doc":"Returns a list of available smart cell definitions.","ref":"Kino.SmartCell.html#definitions/0","title":"Kino.SmartCell.definitions/0","type":"function"},{"doc":"Generates unique variable names with the given prefix. When var_name is nil , allocates and returns the next available name. Otherwise, marks the given suffix as taken, provided that var_name has the given prefix. This function can be used to generate default variable names during smart cell initialization, so that don't overlap.","ref":"Kino.SmartCell.html#prefixed_var_name/2","title":"Kino.SmartCell.prefixed_var_name/2","type":"function"},{"doc":"Converts the given AST to formatted code string.","ref":"Kino.SmartCell.html#quoted_to_string/1","title":"Kino.SmartCell.quoted_to_string/1","type":"function"},{"doc":"Registers a new smart cell. This should usually be called in application.ex when starting the application. Examples Kino.SmartCell . register ( Kino.SmartCell.Custom )","ref":"Kino.SmartCell.html#register/1","title":"Kino.SmartCell.register/1","type":"function"},{"doc":"Invoked whenever the base evaluation context changes. This callback receives the binding and environment available to the smart cell code. Note that this callback runs asynchronously and it receives the PID of the smart cell server, so the result needs to be sent explicitly and handled using Kino.JS.Live.handle_info/2 . Important: remember that data sent between processes is copied, so avoid sending large data structures. In particular, when looking at variables, instead of sending their values, extract and send only the relevant metadata. Important: avoid any heavy work in this callback, as it runs in the same process that evaluates code, so we don't want to block it.","ref":"Kino.SmartCell.html#c:scan_binding/3","title":"Kino.SmartCell.scan_binding/3","type":"callback"},{"doc":"Invoked when the smart cell code is evaluated. This callback receives the result of an evaluation, either the return value or an exception if raised. This callback runs asynchronously and has the same characteristics as scan_binding/3 .","ref":"Kino.SmartCell.html#c:scan_eval_result/2","title":"Kino.SmartCell.scan_eval_result/2","type":"callback"},{"doc":"Invoked to compute the smart cell state as serializable attributes.","ref":"Kino.SmartCell.html#c:to_attrs/1","title":"Kino.SmartCell.to_attrs/1","type":"callback"},{"doc":"Invoked to generate source code based on the given attributes.","ref":"Kino.SmartCell.html#c:to_source/1","title":"Kino.SmartCell.to_source/1","type":"callback"},{"doc":"Checks if the given string is a valid Elixir variable name.","ref":"Kino.SmartCell.html#valid_variable_name?/1","title":"Kino.SmartCell.valid_variable_name?/1","type":"function"},{"doc":"Attributes are an intermediate form of smart cell state, used to persist and restore cells. Attributes are computed using to_attrs/1 and used to generate the source code using to_source/1 . Note that attributes are serialized and deserialized as JSON for persistence, hence make sure to use JSON-friendly data structures. Persisted attributes are passed to Kino.JS.Live.init/2 as the first argument and should be used to restore the relevant state.","ref":"Kino.SmartCell.html#t:attrs/0","title":"Kino.SmartCell.attrs/0","type":"type"},{"doc":"","ref":"Kino.SmartCell.html#t:eval_result/0","title":"Kino.SmartCell.eval_result/0","type":"type"},{"doc":"A number of output formats supported by Livebook.","ref":"Kino.Output.html","title":"Kino.Output","type":"module"},{"doc":"See control/0 .","ref":"Kino.Output.html#control/1","title":"Kino.Output.control/1","type":"function"},{"doc":"See frame/0 .","ref":"Kino.Output.html#frame/2","title":"Kino.Output.frame/2","type":"function"},{"doc":"See image/0 .","ref":"Kino.Output.html#image/2","title":"Kino.Output.image/2","type":"function"},{"doc":"See input/0 .","ref":"Kino.Output.html#input/1","title":"Kino.Output.input/1","type":"function"},{"doc":"Returns text/0 with the inspected term.","ref":"Kino.Output.html#inspect/2","title":"Kino.Output.inspect/2","type":"function"},{"doc":"See js/0 .","ref":"Kino.Output.html#js/1","title":"Kino.Output.js/1","type":"function"},{"doc":"See markdown/0 .","ref":"Kino.Output.html#markdown/1","title":"Kino.Output.markdown/1","type":"function"},{"doc":"Generates a random binary identifier.","ref":"Kino.Output.html#random_ref/0","title":"Kino.Output.random_ref/0","type":"function"},{"doc":"See text/0 .","ref":"Kino.Output.html#text/1","title":"Kino.Output.text/1","type":"function"},{"doc":"A control widget. All controls have the following properties: :type - one of the recognised control types :ref - a unique identifier :destination - the process to send event messages to On top of that, each control type may have additional attributes. Events All control events are sent to :destination as {:event, id, info} , where info is a map including additional details. In particular, it always includes :origin , which is an opaque identifier of the client that triggered the event.","ref":"Kino.Output.html#t:control/0","title":"Kino.Output.control/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:control_attrs/0","title":"Kino.Output.control_attrs/0","type":"type"},{"doc":"Outputs placeholder. Frame with type :default includes the initial list of outputs. Other types can be used to update outputs within the given frame. In all cases the outputs order is reversed, that is, most recent outputs are at the top of the stack.","ref":"Kino.Output.html#t:frame/0","title":"Kino.Output.frame/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:frame_info/0","title":"Kino.Output.frame_info/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:frame_ref/0","title":"Kino.Output.frame_ref/0","type":"type"},{"doc":"An empty output that should be ignored whenever encountered.","ref":"Kino.Output.html#t:ignored/0","title":"Kino.Output.ignored/0","type":"type"},{"doc":"A raw image in the given format.","ref":"Kino.Output.html#t:image/0","title":"Kino.Output.image/0","type":"type"},{"doc":"An input field. All inputs have the following properties: :type - one of the recognised input types :ref - a unique identifier :id - a persistent input identifier, the same on every reevaluation :label - an arbitrary text used as the input caption :default - the initial input value :destination - the process to send event messages to On top of that, each input type may have additional attributes.","ref":"Kino.Output.html#t:input/0","title":"Kino.Output.input/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:input_attrs/0","title":"Kino.Output.input_attrs/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:input_id/0","title":"Kino.Output.input_id/0","type":"type"},{"doc":"JavaScript powered output with dynamic data and events. See Kino.JS and Kino.JS.Live for more details.","ref":"Kino.Output.html#t:js/0","title":"Kino.Output.js/0","type":"type"},{"doc":"Data describing a JS output. Export The :export map describes how the output should be persisted. The output data is put in a Markdown fenced code block. :info_string - used as the info string for the Markdown code block :key - in case the data is a map and only a specific part should be exported","ref":"Kino.Output.html#t:js_info/0","title":"Kino.Output.js_info/0","type":"type"},{"doc":"A JavaScript view definition. JS view is a component rendered on the client side and possibly interacting with a server process within the runtime. :ref - unique identifier :pid - the server process holding the data and handling interactions Assets The :assets map includes information about the relevant files. :archive_path - an absolute path to a .tar.gz archive with all the assets :hash - a checksum of all assets in the archive :js_path - a relative asset path pointing to the JavaScript entrypoint module Communication protocol A client process should connect to the server process by sending: { :connect , pid ( ) , info :: %{ ref : ref ( ) , origin : term ( ) } } And expect the following reply: { :connect_reply , payload , info :: %{ ref : ref ( ) } } The server process may then keep sending one of the following events: { :event , event :: String . t ( ) , payload , info :: %{ ref : ref ( ) } } The client process may keep sending one of the following events: { :event , event :: String . t ( ) , payload , info :: %{ ref : ref ( ) , origin : term ( ) } }","ref":"Kino.Output.html#t:js_view/0","title":"Kino.Output.js_view/0","type":"type"},{"doc":"Markdown content.","ref":"Kino.Output.html#t:markdown/0","title":"Kino.Output.markdown/0","type":"type"},{"doc":"","ref":"Kino.Output.html#t:ref/0","title":"Kino.Output.ref/0","type":"type"},{"doc":"IO text output, adjacent such outputs are treated as a whole","ref":"Kino.Output.html#t:stdout/0","title":"Kino.Output.stdout/0","type":"type"},{"doc":"Livebook cell output may be one of these values and gets rendered accordingly.","ref":"Kino.Output.html#t:t/0","title":"Kino.Output.t/0","type":"type"},{"doc":"Standalone text block.","ref":"Kino.Output.html#t:text/0","title":"Kino.Output.text/0","type":"type"},{"doc":"Protocol defining term formatting in the context of Livebook.","ref":"Kino.Render.html","title":"Kino.Render","type":"protocol"},{"doc":"Transforms the given value into a Livebook-compatible output.","ref":"Kino.Render.html#to_livebook/1","title":"Kino.Render.to_livebook/1","type":"function"},{"doc":"","ref":"Kino.Render.html#t:t/0","title":"Kino.Render.t/0","type":"type"},{"doc":"Conveniences for testing custom Kino components. In practice, Kino.JS.Live kinos communicate with Livebook via the group leader. During tests, Livebook is out of the equation, so we need to mimic this side of the communication. To do so, add the following setup to your test module: import Kino.Test setup :configure_livebook_bridge","ref":"Kino.Test.html","title":"Kino.Test","type":"module"},{"doc":"Asserts a Kino.JS.Live kino will broadcast an event within timeout . Examples assert_broadcast_event ( kino , &quot;bump&quot; , %{ by : 2 } )","ref":"Kino.Test.html#assert_broadcast_event/4","title":"Kino.Test.assert_broadcast_event/4","type":"macro"},{"doc":"Asserts the given output is sent to within timeout . Examples assert_output ( { :markdown , &quot;_hey_&quot; } )","ref":"Kino.Test.html#assert_output/2","title":"Kino.Test.assert_output/2","type":"macro"},{"doc":"Asserts a smart cell update will be broadcasted within timeout . Matches against the source and attribute that are reported as part of the update. Examples assert_smart_cell_update ( kino , %{ &quot;variable&quot; =&gt; &quot;x&quot; , &quot;number&quot; =&gt; 10 } , &quot;x = 10&quot; )","ref":"Kino.Test.html#assert_smart_cell_update/4","title":"Kino.Test.assert_smart_cell_update/4","type":"macro"},{"doc":"","ref":"Kino.Test.html#configure_livebook_bridge/1","title":"Kino.Test.configure_livebook_bridge/1","type":"function"},{"doc":"Connects to a Kino.JS.Live kino and returns the initial data. If resolve_fun is given, it runs after sending the connection request and before awaiting for the reply. Examples data = connect ( kino ) assert data == %{ count : 1 }","ref":"Kino.Test.html#connect/3","title":"Kino.Test.connect/3","type":"function"},{"doc":"Sends a client event to a Kino.JS.Live kino. Examples push_event ( kino , &quot;bump&quot; , %{ &quot;by&quot; =&gt; 2 } )","ref":"Kino.Test.html#push_event/3","title":"Kino.Test.push_event/3","type":"function"},{"doc":"Starts a smart cell defined by the given module. Returns a Kino.JS.Live kino for interacting with the cell, as well as the initial source. Examples { kino , source } = start_smart_cell! ( Kino.SmartCell.Custom , %{ &quot;key&quot; =&gt; &quot;value&quot; } )","ref":"Kino.Test.html#start_smart_cell!/2","title":"Kino.Test.start_smart_cell!/2","type":"function"}]